@@ -29,8 +29,11 @@
 #include <string.h>
 #include <unistd.h>
 #include <signal.h>
+#include <errno.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
+#include <sys/select.h>
+#include <sys/time.h>
 
 #define TOOLTIP_COL		"#fffacd"
 #define TOOLTIP_FONT		"lucidasans-12"
@@ -116,7 +119,7 @@ void
 set_mwm_hints (Window w, PropMwmHints * hints)
 {
 #ifdef DEBUG
-	printf ("set_mwm_hints");
+	printf ("set_mwm_hints\n");
 #endif
 
 	XChangeProperty (dpy, w, motif, motif, 32, PropModeReplace,
@@ -214,7 +217,7 @@ create_icon (char *icon_string)
 			XRaiseWindow (dpy, i->window);
 		}
 	else
-		print_error ("Xpm file not found.", False);
+		print_error ("Xpm file not found.\n", False);
 }
 
 Icon *
@@ -409,9 +412,7 @@ event_button_release (XButtonEvent * ev)
 	if (active_icon)
 		{
 			if (ev->window == active_icon->window && ev->button == Button1
-					&& (ev->state == Button1Mask
-							|| ev->state == (Button1Mask | LockMask)
-							|| ev->state == (Button1Mask | ShiftMask)))
+					&& (ev->state & Button1Mask))
 				fork_exec (active_icon->command);
 		}
 }
@@ -531,6 +532,32 @@ initialize ()
 	parse_rc (getenv ("HOME"), RCFILE);
 }
 
+void
+process_all_events ()
+{
+	while (XPending (dpy))
+		{
+			XEvent ev;
+
+			XNextEvent (dpy, &ev);
+			switch (ev.type)
+				{
+				case EnterNotify:
+					event_enter_notify (&ev.xcrossing);
+					break;
+				case LeaveNotify:
+					event_leave_notify ();
+					break;
+				case ButtonRelease:
+					event_button_release (&ev.xbutton);
+					break;
+				case ConfigureNotify:
+					event_configure_notify (&ev.xconfigure);
+					break;
+				}
+		}
+}
+
 /*
  *
  * Main
@@ -540,40 +567,60 @@ initialize ()
 int
 main (int argc, char *argv[])
 {
+	fd_set readset;
+	int x_fd;
+	int nfds;
+	struct timeval timeout;
+
 	initialize ();
+	process_all_events ();
+
+	x_fd = XConnectionNumber (dpy);
 
-	/* Event loop */
 	while (1)
 		{
-			XEvent ev;
+			FD_ZERO (&readset);
+			FD_SET (x_fd, &readset);
 
-			while (XPending (dpy))
+			/* compute the number of the highest fd for the select call */
+			nfds = x_fd+1;
+
+			timeout.tv_sec=1;
+			timeout.tv_usec=0;
+
+#ifdef DEBUG
+			printf("Calling select. x_fd=%d\n", x_fd);
+#endif
+
+INT:	if (-1 == select (nfds, &readset, NULL, NULL, &timeout))
 				{
-					XNextEvent (dpy, &ev);
-					switch (ev.type)
+					if (errno == EINTR)
 						{
-						case EnterNotify:
-							event_enter_notify (&ev.xcrossing);
-							break;
-						case LeaveNotify:
-							event_leave_notify ();
-							break;
-						case ButtonRelease:
-							event_button_release (&ev.xbutton);
-							break;
-						case ConfigureNotify:
-							event_configure_notify (&ev.xconfigure);
-							break;
-						}
-				}
+							if (do_update)
+								{
+									free_icons ();
+									parse_rc (getenv ("HOME"), RCFILE);
+								}
 
-			if (do_update)
+							/* EINTR means, we just can re-execute the select call.
+							 * Sorry, but in this case just the best is a goto.
+							 */
+							goto INT;
+						}
+
+					/* now comes the case, select fails, and errno is not EINTR. */
+					else
+						return 1;
+				}
+			else
 				{
-					free_icons ();
-					parse_rc (getenv ("HOME"), RCFILE);
+					/* now lets check the fd's */
+					if (FD_ISSET (x_fd, &readset))
+						/* process all pending requests */
+						process_all_events ();
 				}
-			usleep (250);
 		}
+
 	return 0;
 }
 
